;;; default-modes.el --- Sets default mode configurations.
;;;
;;; Commentary:
;;;
;;; This package breaks out mode setting commands that are general
;;; to the configuration of emacs itself. Other specific mode
;;; settings are placed in a corresponding elisp file in this
;;; folder.

(ivy-mode 1)

(setq ivy-use-virtual-buffers t)
(setq enable-recursive-minibuffers t)

;; Configure ivy to be similar to flx-ido.
(setq ivy-re-builders-alist
      '((t . ivy--regex-plus)))

(setq ivy-initial-inputs-alist nil)

;; Enable column numbers
(column-number-mode 1)

;; Enable semantic for language-aware editing commands
(setq semantic-mode 't)

;; Enable company mode
(add-hook 'after-init-hook 'global-company-mode)

;; Globally enable flycheck
(add-hook 'after-init-hook #'global-flycheck-mode)

;; LSP
(use-package lsp-mode
    :ensure t
    :hook ((python-mode . lsp-deferred)
	   (c-mode . lsp-deferred)
	   (ruby-mode . lsp-deferred))
    :commands (lsp lsp-deferred))


(use-package lsp-ui
    :ensure t
    :hook (lsp-mode . lsp-ui-mode))
