#!/bin/zsh

if hash vim 2>/dev/null; then
	export EDITOR=vim
else
	export EDITOR=vi
fi

if hash clang 2>/dev/null; then
	export CC=$(which clang)
else
	export CC=$(which gcc)
fi

if hash clang++ 2>/dev/null; then
	export CXX=$(which clang++)
else
	export CXX=$(which g++)
fi

export OPENSSL_ROOT_DIR=$(which openssl)

# Add openJDK to path
export PATH="/usr/local/opt/openjdk/bin:$PATH"

# Ignore duplicates in history to make history less of a hassle to use.
export HISTCONTROL=ignoredups

# If go is installed, set the $GOPATH to the projects directory created above.
if which go > /dev/null; then
export GOPATH=$HOME/.go
export PATH=${PATH}:`go env GOPATH`/bin
fi

if [ -d "$HOME/.rvm" ]; then
	export PATH="$PATH:$HOME/.rvm/bin"
fi

if [ -f "$HOME/.cargo/env" ]; then
	. "$HOME/.cargo/env"
fi

if [[ "$OSTYPE" == "linux-gnu" ]]; then
	# If the Go install guide was followed for Linux
	# /usr/local/go will contain the go binary.
	export PATH=$PATH:/usr/local/go/bin
fi

if [[ "$OSTYPE" == "darwin"* ]]; then
  # Enable homebrew openssl support
  export LIBRARY_PATH=$LIBRARY_PATH:/usr/local/opt/openssl/lib/
  export PATH="/usr/local/opt/llvm/bin:$PATH"
  export PATH="/usr/local/opt/qt/bin:$PATH"
  export PATH="/usr/local/opt/tcl-tk/bin:$PATH"
fi

export PATH=$HOME/bin:/usr/local/bin:$PATH

